/****************************************************************************
 ****************************************************************************
 **
 **   File: callbacks.c
 **
 **   Description: Callback routines for Xstation monitor.
 **
 ****************************************************************************
 ****************************************************************************/

/************************************************************
*                                                           *
*  Revision history:                                        *
*                                                           *
*  02/03/91  Gary Goldman, IBM Austin                       *
*                                                           *
*************************************************************/

#include "xmonitor.h"
#include <Xm/Text.h>
#include "xsstat.h"

/*****************************************************************************
 *                        Globals
 *****************************************************************************/
extern Bool print;
extern int sample_count;

/*****************************************************************************
 *                        Locals
 *****************************************************************************/
static Bool monitoring_active = FALSE;
static int sample_rate = ON_DEMAND;
static int sample_time_store[2];
static XtIntervalId sample_interval_id = 0;

/*****************************************************************************
 *          		Function Declarations
 *****************************************************************************/
extern int  conv_x_coord();
extern int  conv_y_coord();
extern void init_XSSTAT();
extern void draw_axes();
extern void shift_graph();
extern void draw_graph();
extern void erase_line();
extern void set_counter();

/***************************************************************************
 * Exit the application.
 ***************************************************************************/
void exit_proc(w, tag, reason)
    Widget w;
    int *tag;
    unsigned long *reason;
{
    if (tag != NULL) printf("%s\n", tag);
    exit(0);
}

/***************************************************************************
 * Redraw graph do to expose.
 ***************************************************************************/
void expose_graph(w, graph_num, callback_data)
    Widget w;
    int graph_num;
    XmDrawingAreaCallbackStruct *callback_data;
{
	draw_graph(graph_num, FALSE);
}

/***************************************************************************
 * Act on form widget exposure.
 ***************************************************************************/
void expose_form(w, graph_num, callback_data)
    Widget w;
    int graph_num;
    XmDrawingAreaCallbackStruct *callback_data;
{
	draw_axes(graph_num);
}

/***************************************************************************
 * Scroll graph viewport.
 ***************************************************************************/
void set_scroll_lock(w, tag, reason)
    Widget w;
    int tag;
    unsigned long *reason;
{
	monitor_options.slock = tag;
}

/***************************************************************************
 * Scroll graph viewport.
 ***************************************************************************/
void scroll_graph(w, tag, callback_data)
    Widget w;
    int *tag;
    XmScrollBarCallbackStruct *callback_data;
{
	LINE *line_ptr;
	int graph_num, line_idx, direction, start, end, j;

	graph_num = tag[0];
	direction = tag[1];

	draw_graph(graph_num, TRUE); /* erase lines */
	if (direction == HORIZONTAL) {
		graph[graph_num].view.start.x = callback_data->value;
		graph[graph_num].view.end.x = 
			graph[graph_num].view.start.x+graph[graph_num].view.size.x;
	}
	else {
		graph[graph_num].view.start.y = callback_data->value;
		graph[graph_num].view.end.y = 
				graph[graph_num].view.start.y+graph[graph_num].view.size.y;
	}
	FOR_LINES_IN_GRAPH(line_idx, graph_num) {
		line_ptr = graph[graph_num].line[line_idx];
		start = line_ptr->start_of_data;
		end = graph[graph_num].view.current;
		if (direction == HORIZONTAL)
			for (j = start; j <= end; j++)
				line_ptr->point[j].x = conv_x_coord(j, graph_num);
		else if (direction == VERTICAL) {
			for (j = start; j <= end; j++)
				line_ptr->point[j].y = 
						conv_y_coord(line_ptr->value[j], graph_num);
		}
	}
	draw_graph(graph_num, FALSE);
	if (direction == VERTICAL) set_graph_labels(graph_num);
}

/***************************************************************************
 * Display a graph.
 ***************************************************************************/
void display_graph(w, graph_num, reason)
	Widget w;
	int graph_num;
	unsigned long *reason;
{
	if (!XtIsManaged(graph[graph_num].dialog)) 
			XtManageChild(graph[graph_num].dialog);
}

/***************************************************************************
 * Remove a graph from the display.
 ***************************************************************************/
void remove_graph(w, graph_num, reason)
	Widget w;
	int graph_num;
	unsigned long *reason;
{
	XtUnmanageChild(graph[graph_num].dialog);
}

/***************************************************************************
 * Start up the monitor.
 ***************************************************************************/
void activate_monitor()
{
	GRAPH *graph_ptr;
	int line_num, graph_num, time;

	init_XSSTAT();
	monitoring_active = TRUE;
	sample_count = -1;

	FOR_ALL_GRAPHS(graph_num) {
		XClearWindow(dpy, XtWindow(graph[graph_num].d_area));
		graph[graph_num].view.current = 
			graph[graph_num].geom.max_inc.x-graph[graph_num].view.size.x-1;
	}
	FOR_ALL_LINES(line_num) {
		graph_ptr = line[line_num].graph;
		line[line_num].start_of_data = 
					graph_ptr->geom.max_inc.x-graph_ptr->view.size.x;
	}

	if (monitor_options.sample_rate == ON_DEMAND) {
		XtManageChild(monitor_sample_dialog);
	}
	else if (monitor_options.sample_rate == TIMED_INTERVAL) {
		time = ((monitor_options.sample_time[0]*60)+ 
				 monitor_options.sample_time[1])*1000;
		sample_interval_id = XtAppAddTimeOut(toplevel_context, time, 
							 sample_stats, NULL);
		XtManageChild(monitor_in_progress_dialog);
	}
}

/***************************************************************************
 * Stop the monitor.
 ***************************************************************************/
void stop_monitor(w, tag, reason)
	Widget w;
	int tag;
	unsigned long *reason;
{
	if (tag <= 1) {
		XtUnmanageChild(monitor_in_progress_dialog);
		if (tag) return;
		if (sample_interval_id) {
			XtRemoveTimeOut(sample_interval_id);
			sample_interval_id = 0;
		}
	}
	else {
		XtUnmanageChild(monitor_sample_dialog);
	}
	XSSTATSetStatsOff(monitor_options.server.dpy);
	monitoring_active = FALSE;
}

/***************************************************************************
 * Sample statistics.
 ***************************************************************************/
XtTimerCallbackProc sample_stats()
{
	struct SSTATinfo statistics;
	Bool StatsOn = TRUE;
	LINE *line_ptr;
	GROUP *group_ptr;
	int line_idx, graph_num, current, time;
	unsigned long mask = 255;  /* get all stats available - filter on draw */

	XSSTATGetServerStats(monitor_options.server.dpy, &mask, &statistics, 
						 &StatsOn);
	set_counter(++sample_count);
	FOR_ALL_GRAPHS(graph_num) {
		if (graph[graph_num].view.current == graph[graph_num].geom.max_inc.x) 
			shift_graph(graph_num);
		current = ++graph[graph_num].view.current;
		group_ptr = graph[graph_num].group;
		group_ptr->value_method(statistics, graph_num, current);
		FOR_LINES_IN_GRAPH(line_idx, graph_num) {
			line_ptr = graph[graph_num].line[line_idx];
			line_ptr->point[current].x = conv_x_coord(current, graph_num);
			line_ptr->point[current].y = conv_y_coord(line_ptr->value[current], 
													 graph_num);
			if (print) /* diagnostics only */
				printf("line%d   value = %d   y = %d\n", line_idx, 
						line_ptr->value[current], line_ptr->point[current].y);
		}
		draw_graph(graph_num, FALSE);
	}
	time = ((monitor_options.sample_time[0]*60)+ 
				 monitor_options.sample_time[1])*1000;
	if (monitor_options.sample_rate == TIMED_INTERVAL && monitoring_active) 
		sample_interval_id = XtAppAddTimeOut(toplevel_context, time, 
							 sample_stats, NULL);
}

/***************************************************************************
 * Sets server to be sampled.
 ***************************************************************************/
void set_server(w, tag, reason)
    Widget w;
    int tag;
    unsigned long *reason;
{
	monitor_options.server.current = tag;
}

/***************************************************************************
 * Sets line toggle to monitor or not.
 ***************************************************************************/
void set_line_monitor(w, line_ptr, callback_data)
    Widget w;
    LINE *line_ptr;
    XmToggleButtonCallbackStruct *callback_data;
{ 
	GRAPH *graph_ptr;
	int line_idx;

	line_ptr->monitor = callback_data->set;
	graph_ptr = line_ptr->graph;	
	if (!callback_data->set) {
		if (monitoring_active) erase_line(line_ptr);
		/* if all lines in a graph are toggled off get rid of it */
		FOR_LINES_IN_GRAPH_PTR(line_idx, graph_ptr) {
			if (graph_ptr->line[line_idx]->monitor) break;
		}
		if (line_idx == graph_ptr->num_lines && 
			XtIsManaged(graph->dialog))
				XtUnmanageChild(graph_ptr->dialog);
	}
	else {
		/* make sure the graph is displayed which owns this line */
		if (!XtIsManaged(graph_ptr->dialog))
			XtManageChild(graph_ptr->dialog);
	}
}

/***************************************************************************
 * Sets sampling rate to timed or on demand.
 ***************************************************************************/
void set_sample_rate(w, tag, reason)
    Widget w;
    int tag;
    unsigned long *reason;
{
	int i, time;

	if (!tag) {
		monitor_options.sample_rate = ON_DEMAND;
		if (monitoring_active) {
			if (XtIsManaged(monitor_in_progress_dialog))
				XtUnmanageChild(monitor_in_progress_dialog);
			XtManageChild(monitor_sample_dialog);
		}
	}
	else {
		monitor_options.sample_rate = TIMED_INTERVAL;
    	if (monitoring_active) {
            if (XtIsManaged(monitor_sample_dialog))
           	    XtUnmanageChild(monitor_sample_dialog);
            if (!XtIsManaged(monitor_in_progress_dialog))
	           	XtManageChild(monitor_in_progress_dialog);
			time = ((monitor_options.sample_time[0]*60)+ 
					 monitor_options.sample_time[1])*1000;
			sample_interval_id = XtAppAddTimeOut(toplevel_context, time, 
													 sample_stats, NULL);
        }
		if (!XtIsManaged(sample_rate_dialog)) {
			XtManageChild(sample_rate_dialog);
			for (i = 0; i < 2; i++) 
				sample_time_store[i] = monitor_options.sample_time[i];
		}
	}
}

/***************************************************************************
 * Sets interval scaler changes requested.
 ***************************************************************************/
void set_sample_interval(w, tag, reason)
    Widget w;
    int tag;
    unsigned long *reason;
{
	int i;

	if (tag ==2) {
		/* Cancel */
		XtUnmanageChild(sample_rate_dialog);
		for (i = 0; i < 2; i++) {
			monitor_options.sample_time[i] = sample_time_store[i];
			XmScaleSetValue(sample_rate_scale[i], 
							monitor_options.sample_time[i]);
		}
	}
	else {
		for (i = 0; i < 2; i++)
			XmScaleGetValue(sample_rate_scale[i], 
							&monitor_options.sample_time[i]);
		if (!tag) 
			XtUnmanageChild(sample_rate_dialog); /* OK */
	}
}

/***************************************************************************
 * Verifys and loads history size.
 ***************************************************************************/
void set_history_size(w, tag, callback_data)
    Widget w;
    int tag;
    XmTextVerifyCallbackStruct *callback_data;
{
	/* check for type error first */
	if (!isdigit(*callback_data->text->ptr++))
		printf("entry ERROR\n");
	else
    	monitor_options.history_size = atoi(XmTextGetString(w));
}

/***************************************************************************
 * Sets client filtering stuff.
 ***************************************************************************/
void set_client_filter(w, tag, reason)
    Widget w;
    int tag;
    unsigned long *reason;
{
	Arg option_arg;

	if (tag < 2) {
		monitor_options.client_filter = tag;
	}
	else {
		XtManageChild(client_filter_dialog);
		XtSetArg(option_arg, XmNmenuHistory, 
				 client_filter_widget[monitor_options.client_filter]);
		XtSetValues(client_filter_widget[2], &option_arg, 1);
	}
}

/***************************************************************************
 * Display X protocol toggles.
 ***************************************************************************/
void activate_protocols()
{
    XtManageChild(protocols_dialog);
}

/***************************************************************************
 * Removes X protocol toggles.
 ***************************************************************************/
void close_protocols()
{
    XtUnmanageChild(protocols_dialog);
}
